function C = coding(imageFile, detectorFile)
% CODING computes coding map C for a given image file IMAGEFILE and a
% corresponding keypoint file DETECTORFILE.
%
% See http://www.robots.ox.ac.uk/~vgg/research/affine/detectors.html for a
% desciption of such a keypoint file.
%
%
% For details about the method, refer to
% @article{dickscheid*10:coding,
%   author = {Dickscheid, T. and Schindler, F. and F\"{o}rstner, W.}, 
%   title = {Coding Images with Local Features},
%   journal = {International Journal of Computer Vision}, 
%    year = {2010}, 
%    doi = {10.1007/s11263-010-0340-z}, 
% } 
% If you use this software for your research, we would be glad if you 
% cite our work.
%
% See also: entropy, distance
%
% Licence:
%   For internal use only.
%
% Author:
%   Falko Schindler (falko.schindler@uni-bonn.de)
%   Department of Photogrammetry
%   Institute of Geodesy and Geoinformation
%   University of Bonn
%   Bonn, Germany
%
% Example:
%   C = coding('cameraman.tif', 'sfop.dat');
%   imagesc(C(33 : end - 32, 33 : end - 32));
%
% Copyright 2009-2010

%% load features
[feat, factor] = loadFeatures(detectorFile);

%% prepare coding map
info = imfinfo(imageFile);
imgSize = [info.Height, info.Width];

%% compute coding
if isempty(feat)
    % no feature found
	C = zeros(imgSize(1 : 2));
else
    % initialize vectorized map
    c = zeros(prod(imgSize(1 : 2)), 1);
    % extract feature information from feature list
    [X, Y] = meshgrid(1 : imgSize(2), 1 : imgSize(1));
    [x, y] = deal(X(:), Y(:));
    [xm, ym] = deal(feat(:, 1) + 1, feat(:, 2) + 1);
    [Inv11, Inv12, Inv22] = deal(feat(:, 3), feat(:, 4), feat(:, 5));
    detInv = Inv11 .* Inv22 - Inv12.^2;
    normalization = sqrt(detInv) ./ (2 * pi * factor);
    % distinguish simple isotropic features and anisotropic ones for speed
    if all(Inv11 == Inv22) && all(Inv12 == 0)
        % isotropic keypoints only
        if ~any(mod([xm; ym], 1))
            % integer coordinates only
            C = accumarray([ym, xm], 1, imgSize(1 : 2));
            C = gaussFFT(C, sqrt(1 ./ Inv11 + 5^2), 'G'); %#ok<NASGU>
        else
            % subpixel coordinates
            for f = find(all(~isnan(feat')))
                [x_, y_] = deal(x - xm(f), y - ym(f));
                c = c + exp(- .5 * (x_.^2 + y_.^2) .* Inv11(f)) * normalization(f);
            end
            C = gaussFFT(reshape(c, imgSize(1 : 2)), 5, 'G'); %#ok<NASGU>
        end
    else
        % anisotropic keypoints
        for f = find(all(~isnan(feat')))
            [x_, y_] = deal(x - xm(f), y - ym(f));
            c = c + exp(- .5 * x_.^2    .* Inv11(f) ...
                        -      x_ .* y_ .* Inv12(f) ...
                        - .5 * y_.^2    .* Inv22(f)) * normalization(f);
        end
        C = gaussFFT(reshape(c, imgSize(1 : 2)), 5, 'G'); %#ok<NASGU>
    end
end
