function H = entropy(img)
% ENTROPY computes entropy map H for a given image IMG.
%
% See also: coding, distance
%
%
% For details about the method, refer to
% @article{dickscheid*10:coding,
%   author = {Dickscheid, T. and Schindler, F. and F\"{o}rstner, W.}, 
%   title = {Coding Images with Local Features},
%   journal = {International Journal of Computer Vision}, 
%    year = {2010}, 
%    doi = {10.1007/s11263-010-0340-z}, 
% } 
% If you use this software for your research, we would be glad if you 
% cite our work.
%
% Licence:
%   For internal use only.
%
% Author:
%   Falko Schindler (falko.schindler@uni-bonn.de)
%   Department of Photogrammetry
%   Institute of Geodesy and Geoinformation
%   University of Bonn
%   Bonn, Germany
%
% Example:
%   H = entropy(imread('cameraman.tif'));
%   imagesc(H(33 : end - 32, 33 : end - 32));
%
% Copyright 2009-2010

%% set parameters
% patch size
N = 1 + 2.^(1 : 6);
% gray-scale resolution
epsilon = 1 / 256;
% noise variance
sigma_n = 0;

%% sum entropy for each patch size at each off-boundary pixel
% initialize entropy map
H = zeros(size(img));
% process each patch size
for b = (N - 1) / 2, fprintf('_'); end, fprintf('\n');
for b = (N - 1) / 2;
    % compute boundary width
    maxB = max(N - 1) / 2;
    % process each pixel within boundary
    for r = maxB + 1 : size(img, 1) - maxB
        for c = maxB + 1 : size(img, 2) - maxB
            % current patch
            g = img(r - b : r + b, c - b : c + b);
            % patch size
            N2 = numel(g);
            % noise variance + rounding error
            Vn = 2 * (epsilon^2 / 12 + sigma_n^2);
            % regularized power spectrum
            P = max(dct2(g).^2 - Vn, 0) / N2;
            % increment entropy at current pixel
            H(r, c) = H(r, c) + ...
                sum(max(0, log((P(2 : N2) + eps) / Vn * N2))) / ...
                log(2) / N2 / 2;
        end
    end
    fprintf('''');
end
fprintf('\n');
